package twirperr

//go:generate eventbus-mockery -dir . -output ./mocks -case snake -name "DBError"
type DBError interface {
	error

	DuplicateIAMRoleARN() bool
	DuplicateIAMRoleLabelForService() bool
	DuplicateServiceCatalogID() bool
	DuplicateSubscriptionTargetQueueURL() bool

	AuthorizedFieldNotFound() bool
	AuthorizedFieldPublisherGrantNotFound() bool
	AuthorizedFieldSubscriberGrantNotFound() bool
	EventStreamNotFound() bool
	EventTypeNotFound() bool
	IAMRoleNotFound() bool
	SubscriptionTargetNotFound() bool
	ServiceNotFound() bool
}
